@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_shore.c,v 1.24 2006/10/27 23:45:23 pwessel Exp $
+ *	$Id: gmt_shore.c,v 1.39 2009/02/16 20:58:10 guru Exp $
  *
- *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -56,7 +56,9 @@ void GMT_br_to_degree (struct GMT_BR *c, short int dx, short int dy, double *lon
 void shore_prepare_sides(struct GMT_SHORE *c, int dir);
 int GMT_shore_asc_sort (const void *a, const void *b);
 int GMT_shore_desc_sort(const void *a, const void *b);
-BOOLEAN shore_getpathname (char *name, char *path);
+char *GMT_shore_getpathname (char *name, char *path);
+void GMT_shore_check (BOOLEAN ok[5]);
+int GMT_res_to_int (char res);
 
 int GMT_set_resolution (char *res, char opt)
 {
@@ -90,53 +92,89 @@ int GMT_set_resolution (char *res, char opt)
 	return (base);
 }
 
-int GMT_init_shore (char res, struct GMT_SHORE *c, double w, double e, double s, double n)
-/* res: Resolution (f, h, i, l, c */
-                
+void GMT_shore_check (BOOLEAN ok[5])
+/* Sets ok to TRUE for those resolutions available in share for
+ * resolution (f, h, i, l, c) */
 {
-	int i, nb, idiv, iw, ie, is, in, this_south, this_west;
+	int i, j, n_found;
+	char stem[GMT_TEXT_LEN], path[BUFSIZ], *res = "clihf", *kind[3] = {"GSHHS", "river", "border"};
+	
+	for (i = 0; i < 5; i++) {	/* For each resolution... */
+		ok[i] = FALSE;
+		for (j = n_found = 0; j < 3; j++) {	/* For each data type... */
+			sprintf (stem, "binned_%s_%c", kind[j], res[i]);
+	        	if (!GMT_shore_getpathname (stem, path)) continue;	/* Failed to find file */
+			n_found++;	/* Increment how many found so far for this resolution */
+		}
+		ok[i] = (n_found == 3);	/* Need all three sets to say this resolution is covered */
+	}
+}
+
+int GMT_res_to_int (char res)
+{	/* Turns a resolution letter into a 0-4 integer */
+	int i, j;
+	char *type = "clihf";
+	
+	for (i = -1, j = 0; i == -1 && j < 5; j++) if (res == type[j]) i = j;
+	return (i);
+}
+
+char GMT_shore_adjust_res (char res) {	/* Returns the highest available resolution <= to specified resolution */
+	int k, orig;
+	BOOLEAN ok[5];	
+	char *type = "clihf";
+	(void)GMT_shore_check (ok);		/* See which resolutions we have */
+	k = orig = GMT_res_to_int (res);	/* Get integer value of requested resolution */
+	while (!ok[k] && k >= 0) k--;		/* Drop down one level to see if we have a lower resolution available */
+	if (k >= 0 && k != orig) fprintf (stderr, "%s: Warning: Resolution %c not available, substituting resolution %c\n", GMT_program, res, type[k]);
+	return ((k == -1) ? res : type[k]);	/* Return the chosen resolution */
+}
+
+int GMT_init_shore (char res, struct GMT_SHORE *c, double w, double e, double s, double n) {	/* res: Resolution (f, h, i, l, c */
+	int i, nb, idiv, iw, ie, is, in, this_south, this_west, err;
 	short *stmp;
 	int *itmp;
 	size_t start[1], count[1];
-	char file[GMT_TEXT_LEN], path[BUFSIZ];
+	char stem[GMT_TEXT_LEN], path[BUFSIZ];
 	
-	sprintf (file, "binned_GSHHS_%c.cdf", res);
+	sprintf (stem, "binned_GSHHS_%c", res);
 	
-        if (!shore_getpathname (file, path)) return (-1);	/* Failed to find file */
-        
-	check_nc_status (nc_open (path, NC_NOWRITE,&c->cdfid));
+	if (!GMT_shore_getpathname (stem, path)) return (GMT_GRDIO_FILE_NOT_FOUND);	/* Failed to find file */
+	memset ((void *)c, 0, sizeof (struct GMT_SHORE));
+		
+	GMT_err_trap (nc_open (path, NC_NOWRITE,&c->cdfid));
                 
 	/* Get all id tags */
-	check_nc_status (nc_inq_varid (c->cdfid, "Bin_size_in_minutes", &c->bin_size_id));
-        check_nc_status (nc_inq_varid (c->cdfid, "N_bins_in_360_longitude_range", &c->bin_nx_id));
-        check_nc_status (nc_inq_varid (c->cdfid, "N_bins_in_180_degree_latitude_range", &c->bin_ny_id));
-        check_nc_status (nc_inq_varid (c->cdfid, "N_bins_in_file", &c->n_bin_id));
-        check_nc_status (nc_inq_varid (c->cdfid, "N_segments_in_file", &c->n_seg_id));
-        check_nc_status (nc_inq_varid (c->cdfid, "N_points_in_file", &c->n_pt_id));
-        check_nc_status (nc_inq_varid (c->cdfid, "Id_of_first_segment_in_a_bin", &c->bin_firstseg_id));
-        check_nc_status (nc_inq_varid (c->cdfid, "Embedded_node_levels_in_a_bin", &c->bin_info_id));
-        check_nc_status (nc_inq_varid (c->cdfid, "N_segments_in_a_bin", &c->bin_nseg_id));
-        check_nc_status (nc_inq_varid (c->cdfid, "Embedded_npts_levels_exit_entry_for_a_segment", &c->seg_info_id));
-        check_nc_status (nc_inq_varid (c->cdfid, "Ten_times_the_km_squared_area_of_the_parent_polygon_of_a_segment", &c->seg_area_id));
-        check_nc_status (nc_inq_varid (c->cdfid, "Id_of_first_point_in_a_segment", &c->seg_start_id));
-        check_nc_status (nc_inq_varid (c->cdfid, "Relative_longitude_from_SW_corner_of_bin", &c->pt_dx_id));
-        check_nc_status (nc_inq_varid (c->cdfid, "Relative_latitude_from_SW_corner_of_bin", &c->pt_dy_id));
+	GMT_err_trap (nc_inq_varid (c->cdfid, "Bin_size_in_minutes", &c->bin_size_id));
+        GMT_err_trap (nc_inq_varid (c->cdfid, "N_bins_in_360_longitude_range", &c->bin_nx_id));
+        GMT_err_trap (nc_inq_varid (c->cdfid, "N_bins_in_180_degree_latitude_range", &c->bin_ny_id));
+        GMT_err_trap (nc_inq_varid (c->cdfid, "N_bins_in_file", &c->n_bin_id));
+        GMT_err_trap (nc_inq_varid (c->cdfid, "N_segments_in_file", &c->n_seg_id));
+        GMT_err_trap (nc_inq_varid (c->cdfid, "N_points_in_file", &c->n_pt_id));
+        GMT_err_trap (nc_inq_varid (c->cdfid, "Id_of_first_segment_in_a_bin", &c->bin_firstseg_id));
+        GMT_err_trap (nc_inq_varid (c->cdfid, "Embedded_node_levels_in_a_bin", &c->bin_info_id));
+        GMT_err_trap (nc_inq_varid (c->cdfid, "N_segments_in_a_bin", &c->bin_nseg_id));
+        GMT_err_trap (nc_inq_varid (c->cdfid, "Embedded_npts_levels_exit_entry_for_a_segment", &c->seg_info_id));
+        GMT_err_trap (nc_inq_varid (c->cdfid, "Ten_times_the_km_squared_area_of_the_parent_polygon_of_a_segment", &c->seg_area_id));
+        GMT_err_trap (nc_inq_varid (c->cdfid, "Id_of_first_point_in_a_segment", &c->seg_start_id));
+        GMT_err_trap (nc_inq_varid (c->cdfid, "Relative_longitude_from_SW_corner_of_bin", &c->pt_dx_id));
+        GMT_err_trap (nc_inq_varid (c->cdfid, "Relative_latitude_from_SW_corner_of_bin", &c->pt_dy_id));
 
 	/* Get attributes */
-	check_nc_status (nc_get_att_text (c->cdfid, c->pt_dx_id, "units", c->units));
-        check_nc_status (nc_get_att_text (c->cdfid, NC_GLOBAL, "title", c->title));
-        check_nc_status (nc_get_att_text (c->cdfid, NC_GLOBAL, "source", c->source));
+	GMT_err_trap (nc_get_att_text (c->cdfid, c->pt_dx_id, "units", c->units));
+        GMT_err_trap (nc_get_att_text (c->cdfid, NC_GLOBAL, "title", c->title));
+        GMT_err_trap (nc_get_att_text (c->cdfid, NC_GLOBAL, "source", c->source));
 
 	/* Get global variables */
 
 	start[0] = 0;
 
-	check_nc_status (nc_get_var1_int (c->cdfid, c->bin_size_id, start, &c->bin_size));
-        check_nc_status (nc_get_var1_int (c->cdfid, c->bin_nx_id, start, &c->bin_nx));
-        check_nc_status (nc_get_var1_int (c->cdfid, c->bin_ny_id, start, &c->bin_ny));
-        check_nc_status (nc_get_var1_int (c->cdfid, c->n_bin_id, start, &c->n_bin));
-        check_nc_status (nc_get_var1_int (c->cdfid, c->n_seg_id, start, &c->n_seg));
-        check_nc_status (nc_get_var1_int (c->cdfid, c->n_pt_id, start, &c->n_pt));
+	GMT_err_trap (nc_get_var1_int (c->cdfid, c->bin_size_id, start, &c->bin_size));
+        GMT_err_trap (nc_get_var1_int (c->cdfid, c->bin_nx_id, start, &c->bin_nx));
+        GMT_err_trap (nc_get_var1_int (c->cdfid, c->bin_ny_id, start, &c->bin_ny));
+        GMT_err_trap (nc_get_var1_int (c->cdfid, c->n_bin_id, start, &c->n_bin));
+        GMT_err_trap (nc_get_var1_int (c->cdfid, c->n_seg_id, start, &c->n_seg));
+        GMT_err_trap (nc_get_var1_int (c->cdfid, c->n_pt_id, start, &c->n_pt));
 
 
 	c->scale = (c->bin_size / 60.0) / 65535.0;
@@ -175,31 +213,31 @@ int GMT_init_shore (char res, struct GMT_SHORE *c, double w, double e, double s,
 	count[0] = c->n_bin;
 	stmp = (short *) GMT_memory (VNULL, (size_t)c->n_bin, sizeof (short), "GMT_init_shore");
 	
-	check_nc_status (nc_get_vara_short (c->cdfid, c->bin_info_id, start, count, stmp));
+	GMT_err_trap (nc_get_vara_short (c->cdfid, c->bin_info_id, start, count, stmp));
 	for (i = 0; i < c->nb; i++) c->bin_info[i] = stmp[c->bins[i]];
 	
-        check_nc_status (nc_get_vara_short (c->cdfid, c->bin_nseg_id, start, count, stmp));
+        GMT_err_trap (nc_get_vara_short (c->cdfid, c->bin_nseg_id, start, count, stmp));
 	for (i = 0; i < c->nb; i++) c->bin_nseg[i] = stmp[c->bins[i]];
 	GMT_free ((void *)stmp);
 	
 	itmp = (int *) GMT_memory (VNULL, (size_t)c->n_bin, sizeof (int), "GMT_init_shore");
-        check_nc_status (nc_get_vara_int (c->cdfid, c->bin_firstseg_id, start, count, itmp));
+        GMT_err_trap (nc_get_vara_int (c->cdfid, c->bin_firstseg_id, start, count, itmp));
 	for (i = 0; i < c->nb; i++) c->bin_firstseg[i] = itmp[c->bins[i]];
 	
 	GMT_free ((void *)itmp);
 	
-	return (0);
+	return (GMT_NOERROR);
 }
 
-void GMT_get_shore_bin (int b, struct GMT_SHORE *c, double min_area, int min_level, int max_level)
+int GMT_get_shore_bin (int b, struct GMT_SHORE *c, double min_area, int min_level, int max_level)
 /* b: index number into c->bins */
 /* min_area: Polygons with area less than this are ignored */
 /* min_level: Polygons with lower levels are ignored */
 /* max_level: Polygons with higher levels are ignored */
 {
 	size_t start[1], count[1];
-	int cut_area, *seg_area, *seg_info, *seg_start;
-	int s, i;
+	int *seg_area, *seg_info, *seg_start;
+	int s, i, err, cut_area;
 	double w, e, dx;
 	
 	c->node_level[0] = (unsigned char)MIN (((unsigned short)c->bin_info[b] >> 9) & 7, max_level);
@@ -218,7 +256,7 @@ void GMT_get_shore_bin (int b, struct GMT_SHORE *c, double min_area, int min_lev
 	e = w + dx;
 	c->leftmost_bin = ((w <= project_info.w) && (e > project_info.w));
 
-	if (c->bin_nseg[b] == 0) return;
+	if (c->bin_nseg[b] == 0) return (GMT_NOERROR);
 	
 	cut_area = irint (10.0 * min_area);
 	start[0] = c->bin_firstseg[b];
@@ -227,13 +265,13 @@ void GMT_get_shore_bin (int b, struct GMT_SHORE *c, double min_area, int min_lev
 	seg_area = (int *) GMT_memory (VNULL, (size_t)c->bin_nseg[b], sizeof (int), "GMT_get_shore_bin");
 	seg_info = (int *) GMT_memory (VNULL, (size_t)c->bin_nseg[b], sizeof (int), "GMT_get_shore_bin");
 	seg_start = (int *) GMT_memory (VNULL, (size_t)c->bin_nseg[b], sizeof (int), "GMT_get_shore_bin");
-	
-	check_nc_status (nc_get_vara_int (c->cdfid, c->seg_area_id, start, count, seg_area));
-        check_nc_status (nc_get_vara_int (c->cdfid, c->seg_info_id, start, count, seg_info));
-        check_nc_status (nc_get_vara_int (c->cdfid, c->seg_start_id, start, count, seg_start));
-	
+
+	GMT_err_trap (nc_get_vara_int (c->cdfid, c->seg_area_id, start, count, seg_area));
+        GMT_err_trap (nc_get_vara_int (c->cdfid, c->seg_info_id, start, count, seg_info));
+        GMT_err_trap (nc_get_vara_int (c->cdfid, c->seg_start_id, start, count, seg_start));
+
 	/* First tally how many useful segments */
-	
+
 	for (s = i = 0; i < c->bin_nseg[b]; i++) {
 		if (cut_area > 0 && seg_area[i] < cut_area) continue;
 		if (((seg_info[i] >> 6) & 7) < min_level) continue;
@@ -244,16 +282,16 @@ void GMT_get_shore_bin (int b, struct GMT_SHORE *c, double min_area, int min_lev
 		s++;
 	}
 	c->ns = s;
-	
+
 	if (c->ns == 0) {	/* No useful segments in this bin */
 		GMT_free ((void *) seg_info);	
 		GMT_free ((void *) seg_area);	
 		GMT_free ((void *) seg_start);
-		return;
+		return (GMT_NOERROR);
 	}
-	
+
 	c->seg = (struct GMT_SHORE_SEGMENT *) GMT_memory (VNULL, (size_t)c->ns, sizeof (struct GMT_SHORE_SEGMENT), "GMT_get_shore_bin");
-	
+
 	for (s = 0; s < c->ns; s++) {
 		c->seg[s].level = (seg_info[s] >> 6) & 7;
 		c->seg[s].n = (seg_info[s] >> 9);
@@ -263,68 +301,69 @@ void GMT_get_shore_bin (int b, struct GMT_SHORE *c, double min_area, int min_lev
 		c->seg[s].dy = (short *) GMT_memory (VNULL, (size_t)c->seg[s].n, sizeof (short), "GMT_get_shore_bin");
 		start[0] = seg_start[s];
 		count[0] = c->seg[s].n;
-		check_nc_status (nc_get_vara_short (c->cdfid, c->pt_dx_id, start, count, c->seg[s].dx));
-                check_nc_status (nc_get_vara_short (c->cdfid, c->pt_dy_id, start, count, c->seg[s].dy));	
+		GMT_err_trap (nc_get_vara_short (c->cdfid, c->pt_dx_id, start, count, c->seg[s].dx));
+                GMT_err_trap (nc_get_vara_short (c->cdfid, c->pt_dy_id, start, count, c->seg[s].dy));	
 	}
-		
+
 	GMT_free ((void *) seg_info);	
 	GMT_free ((void *) seg_area);	
 	GMT_free ((void *) seg_start);	
-	
+
+	return (GMT_NOERROR);
 }
 
 int GMT_init_br (char which, char res, struct GMT_BR *c, double w, double e, double s, double n)
 /* which: r(iver) or b(order) */
 /* res: Resolution (f, h, i, l, c */
 {
-	int i, nb, idiv, iw, ie, is, in, this_south, this_west;
+	int i, nb, idiv, iw, ie, is, in, this_south, this_west, err;
 	short *stmp;
 	int *itmp;
 	size_t start[1], count[1];
-	char file[GMT_TEXT_LEN], path[BUFSIZ];
+	char stem[GMT_TEXT_LEN], path[BUFSIZ];
 	
 	if (which == 'r')
-		sprintf (file, "binned_river_%c.cdf", res);
+		sprintf (stem, "binned_river_%c", res);
 	else
-		sprintf (file, "binned_border_%c.cdf", res);
+		sprintf (stem, "binned_border_%c", res);
 	
-        if (!shore_getpathname (file, path)) return (-1);	/* Failed to find file */
+        if (!GMT_shore_getpathname (stem, path)) return (-1);	/* Failed to find file */
 
-	check_nc_status (nc_open (path, NC_NOWRITE, &c->cdfid));
+	GMT_err_trap (nc_open (path, NC_NOWRITE, &c->cdfid));
         
 	/* Get all id tags */
-	check_nc_status (nc_inq_varid (c->cdfid, "Bin_size_in_minutes", &c->bin_size_id));
-        check_nc_status (nc_inq_varid (c->cdfid, "N_bins_in_360_longitude_range", &c->bin_nx_id));
-        check_nc_status (nc_inq_varid (c->cdfid, "N_bins_in_180_degree_latitude_range", &c->bin_ny_id));
-        check_nc_status (nc_inq_varid (c->cdfid, "N_bins_in_file", &c->n_bin_id));
-        check_nc_status (nc_inq_varid (c->cdfid, "N_segments_in_file", &c->n_seg_id));
-        check_nc_status (nc_inq_varid (c->cdfid, "N_points_in_file", &c->n_pt_id));
+	GMT_err_trap (nc_inq_varid (c->cdfid, "Bin_size_in_minutes", &c->bin_size_id));
+        GMT_err_trap (nc_inq_varid (c->cdfid, "N_bins_in_360_longitude_range", &c->bin_nx_id));
+        GMT_err_trap (nc_inq_varid (c->cdfid, "N_bins_in_180_degree_latitude_range", &c->bin_ny_id));
+        GMT_err_trap (nc_inq_varid (c->cdfid, "N_bins_in_file", &c->n_bin_id));
+        GMT_err_trap (nc_inq_varid (c->cdfid, "N_segments_in_file", &c->n_seg_id));
+        GMT_err_trap (nc_inq_varid (c->cdfid, "N_points_in_file", &c->n_pt_id));
          
-        check_nc_status (nc_inq_varid (c->cdfid, "Id_of_first_segment_in_a_bin", &c->bin_firstseg_id));
-        check_nc_status (nc_inq_varid (c->cdfid, "N_segments_in_a_bin", &c->bin_nseg_id));
+        GMT_err_trap (nc_inq_varid (c->cdfid, "Id_of_first_segment_in_a_bin", &c->bin_firstseg_id));
+        GMT_err_trap (nc_inq_varid (c->cdfid, "N_segments_in_a_bin", &c->bin_nseg_id));
          
-        check_nc_status (nc_inq_varid (c->cdfid, "N_points_for_a_segment", &c->seg_n_id));
-        check_nc_status (nc_inq_varid (c->cdfid, "Hierarchial_level_of_a_segment", &c->seg_level_id));
-        check_nc_status (nc_inq_varid (c->cdfid, "Id_of_first_point_in_a_segment", &c->seg_start_id));
+        GMT_err_trap (nc_inq_varid (c->cdfid, "N_points_for_a_segment", &c->seg_n_id));
+        GMT_err_trap (nc_inq_varid (c->cdfid, "Hierarchial_level_of_a_segment", &c->seg_level_id));
+        GMT_err_trap (nc_inq_varid (c->cdfid, "Id_of_first_point_in_a_segment", &c->seg_start_id));
  
-        check_nc_status (nc_inq_varid (c->cdfid, "Relative_longitude_from_SW_corner_of_bin", &c->pt_dx_id));
-        check_nc_status (nc_inq_varid (c->cdfid, "Relative_latitude_from_SW_corner_of_bin", &c->pt_dy_id));
+        GMT_err_trap (nc_inq_varid (c->cdfid, "Relative_longitude_from_SW_corner_of_bin", &c->pt_dx_id));
+        GMT_err_trap (nc_inq_varid (c->cdfid, "Relative_latitude_from_SW_corner_of_bin", &c->pt_dy_id));
 
 	/* Get attributes */
-	check_nc_status (nc_get_att_text (c->cdfid, c->pt_dx_id, "units", c->units));
-        check_nc_status (nc_get_att_text (c->cdfid, NC_GLOBAL, "title", c->title));
-        check_nc_status (nc_get_att_text (c->cdfid, NC_GLOBAL, "source", c->source));
+	GMT_err_trap (nc_get_att_text (c->cdfid, c->pt_dx_id, "units", c->units));
+        GMT_err_trap (nc_get_att_text (c->cdfid, NC_GLOBAL, "title", c->title));
+        GMT_err_trap (nc_get_att_text (c->cdfid, NC_GLOBAL, "source", c->source));
 
 	/* Get global variables */
 
 	start[0] = 0;
 	
-	check_nc_status (nc_get_var1_int (c->cdfid, c->bin_size_id, start, &c->bin_size));
-        check_nc_status (nc_get_var1_int (c->cdfid, c->bin_nx_id, start, &c->bin_nx));
-        check_nc_status (nc_get_var1_int (c->cdfid, c->bin_ny_id, start, &c->bin_ny));
-        check_nc_status (nc_get_var1_int (c->cdfid, c->n_bin_id, start, &c->n_bin));
-        check_nc_status (nc_get_var1_int (c->cdfid, c->n_seg_id, start, &c->n_seg));
-        check_nc_status (nc_get_var1_int (c->cdfid, c->n_pt_id, start, &c->n_pt));
+	GMT_err_trap (nc_get_var1_int (c->cdfid, c->bin_size_id, start, &c->bin_size));
+        GMT_err_trap (nc_get_var1_int (c->cdfid, c->bin_nx_id, start, &c->bin_nx));
+        GMT_err_trap (nc_get_var1_int (c->cdfid, c->bin_ny_id, start, &c->bin_ny));
+        GMT_err_trap (nc_get_var1_int (c->cdfid, c->n_bin_id, start, &c->n_bin));
+        GMT_err_trap (nc_get_var1_int (c->cdfid, c->n_seg_id, start, &c->n_seg));
+        GMT_err_trap (nc_get_var1_int (c->cdfid, c->n_pt_id, start, &c->n_pt));
  
 
 	c->scale = (c->bin_size / 60.0) / 65535.0;
@@ -362,12 +401,12 @@ int GMT_init_br (char which, char res, struct GMT_BR *c, double w, double e, dou
 	count[0] = c->n_bin;
 	stmp = (short *) GMT_memory (VNULL, (size_t)c->n_bin, sizeof (short), "GMT_init_br");
 	
-	check_nc_status (nc_get_vara_short (c->cdfid, c->bin_nseg_id, start, count, stmp));
+	GMT_err_trap (nc_get_vara_short (c->cdfid, c->bin_nseg_id, start, count, stmp));
 	for (i = 0; i < c->nb; i++) c->bin_nseg[i] = stmp[c->bins[i]];
 	GMT_free ((void *)stmp);
 	
 	itmp = (int *) GMT_memory (VNULL, (size_t)c->n_bin, sizeof (int), "GMT_init_br");
-	check_nc_status (nc_get_vara_int (c->cdfid, c->bin_firstseg_id, start, count, itmp));
+	GMT_err_trap (nc_get_vara_int (c->cdfid, c->bin_firstseg_id, start, count, itmp));
 	for (i = 0; i < c->nb; i++) c->bin_firstseg[i] = itmp[c->bins[i]];
 	
 	GMT_free ((void *)itmp);
@@ -375,7 +414,7 @@ int GMT_init_br (char which, char res, struct GMT_BR *c, double w, double e, dou
 	return (0);
 }
 
-void GMT_get_br_bin (int b, struct GMT_BR *c, int *level, int n_levels)
+int GMT_get_br_bin (int b, struct GMT_BR *c, int *level, int n_levels)
 /* b: index number into c->bins */
 /* level: Levels of features to extract */
 /* n_levels: # of such levels. 0 means use all levels */
@@ -383,13 +422,13 @@ void GMT_get_br_bin (int b, struct GMT_BR *c, int *level, int n_levels)
 	size_t start[1], count[1];
 	int *seg_start;
 	short *seg_n, *seg_level;
-	int s, i, k, skip;
+	int s, i, k, skip, err;
 	
 	c->lon_sw = (c->bins[b] % c->bin_nx) * c->bin_size / 60.0;
 	c->lat_sw = 90.0 - ((c->bins[b] / c->bin_nx) + 1) * c->bin_size / 60.0;
 	c->ns = c->bin_nseg[b];
 	
-	if (c->ns == 0) return;
+	if (c->ns == 0) return (GMT_NOERROR);
 	
 	start[0] = c->bin_firstseg[b];
 	count[0] = c->bin_nseg[b];
@@ -398,13 +437,11 @@ void GMT_get_br_bin (int b, struct GMT_BR *c, int *level, int n_levels)
 	seg_level = (short *) GMT_memory (VNULL, (size_t)c->bin_nseg[b], sizeof (short), "GMT_get_br_bin");
 	seg_start = (int *) GMT_memory (VNULL, (size_t)c->bin_nseg[b], sizeof (int), "GMT_get_br_bin");
 	
-	check_nc_status (nc_get_vara_short (c->cdfid, c->seg_n_id, start, count, seg_n));
-        check_nc_status (nc_get_vara_short (c->cdfid, c->seg_level_id, start, count, seg_level));
-        check_nc_status (nc_get_vara_int (c->cdfid, c->seg_start_id, start, count, seg_start));
+	GMT_err_trap (nc_get_vara_short (c->cdfid, c->seg_n_id, start, count, seg_n));
+        GMT_err_trap (nc_get_vara_short (c->cdfid, c->seg_level_id, start, count, seg_level));
+        GMT_err_trap (nc_get_vara_int (c->cdfid, c->seg_start_id, start, count, seg_start));
 
-
-	c->seg = (struct GMT_BR_SEGMENT *) GMT_memory (VNULL, (size_t)c->ns, sizeof (struct GMT_BR_SEGMENT), "GMT_get_br_bin");
-	
+	c->seg = NULL;
 	for (s = i = 0; i < c->ns; i++) {
 		if (n_levels == 0)
 			skip = FALSE;
@@ -413,15 +450,15 @@ void GMT_get_br_bin (int b, struct GMT_BR *c, int *level, int n_levels)
 				if (seg_level[i] == level[k]) skip = FALSE;
 		}
 		if (skip) continue;
-		
+		if (!c->seg) c->seg = (struct GMT_BR_SEGMENT *) GMT_memory (VNULL, (size_t)c->ns, sizeof (struct GMT_BR_SEGMENT), "GMT_get_br_bin");
 		c->seg[s].n = seg_n[i];
 		c->seg[s].level = seg_level[i];
 		c->seg[s].dx = (short *) GMT_memory (VNULL, (size_t)c->seg[s].n, sizeof (short), "GMT_get_br_bin");
 		c->seg[s].dy = (short *) GMT_memory (VNULL, (size_t)c->seg[s].n, sizeof (short), "GMT_get_br_bin");
 		start[0] = seg_start[i];
 		count[0] = c->seg[s].n;
-		check_nc_status (nc_get_vara_short (c->cdfid, c->pt_dx_id, start, count, c->seg[s].dx));
-                check_nc_status (nc_get_vara_short (c->cdfid, c->pt_dy_id, start, count, c->seg[s].dy));
+		GMT_err_trap (nc_get_vara_short (c->cdfid, c->pt_dx_id, start, count, c->seg[s].dx));
+                GMT_err_trap (nc_get_vara_short (c->cdfid, c->pt_dy_id, start, count, c->seg[s].dy));
 
 		s++;
 	}
@@ -432,6 +469,7 @@ void GMT_get_br_bin (int b, struct GMT_BR *c, int *level, int n_levels)
 	GMT_free ((void *) seg_level);	
 	GMT_free ((void *) seg_start);	
 	
+	return (GMT_NOERROR);
 }
 
 int GMT_assemble_shore (struct GMT_SHORE *c, int dir, int first_level, BOOLEAN assemble, BOOLEAN shift, double west, double east, struct GMT_GSHHS_POL **pol)
@@ -565,7 +603,7 @@ int GMT_assemble_shore (struct GMT_SHORE *c, int dir, int first_level, BOOLEAN a
 		p[P].level = (dir == 1) ? 2 * ((low_level - 1) / 2) + 1: 2 * (low_level/2);
 		P++;
 		if (P == p_alloc) {
-			p_alloc += GMT_SMALL_CHUNK;
+			p_alloc <<= 1;
 			p = (struct GMT_GSHHS_POL *) GMT_memory ((void *)p, (size_t)p_alloc, sizeof (struct GMT_GSHHS_POL), "GMT_assemble_shore");
 		}
 		
@@ -583,7 +621,7 @@ int GMT_assemble_shore (struct GMT_SHORE *c, int dir, int first_level, BOOLEAN a
 		p[P].level = c->seg[id].level;
 		P++;
 		if (P == p_alloc) {
-			p_alloc += GMT_SMALL_CHUNK;
+			p_alloc <<= 1;
 			p = (struct GMT_GSHHS_POL *) GMT_memory ((void *)p, (size_t)p_alloc, sizeof (struct GMT_GSHHS_POL), "GMT_assemble_shore");
 		}
 	}
@@ -639,6 +677,7 @@ void GMT_free_br (struct GMT_BR *c)
 		GMT_free ((void *)c->seg[i].dy);
 	}
 	if (c->ns) GMT_free ((void *)c->seg);
+	
 }
 		
 void GMT_shore_cleanup (struct GMT_SHORE *c)
@@ -647,7 +686,7 @@ void GMT_shore_cleanup (struct GMT_SHORE *c)
 	GMT_free ((void *)c->bin_info);
 	GMT_free ((void *)c->bin_nseg);
 	GMT_free ((void *)c->bin_firstseg);
-	check_nc_status (nc_close (c->cdfid));
+	nc_close (c->cdfid);
 }
 
 void GMT_br_cleanup (struct GMT_BR *c)
@@ -655,11 +694,11 @@ void GMT_br_cleanup (struct GMT_BR *c)
 	GMT_free ((void *)c->bins);
 	GMT_free ((void *)c->bin_nseg);
 	GMT_free ((void *)c->bin_firstseg);
-        check_nc_status (nc_close (c->cdfid)); 
+        nc_close (c->cdfid);
 	
 }
 
-int GMT_prep_polygons (struct GMT_GSHHS_POL **p_old, int np, BOOLEAN greenwich, BOOLEAN sample, double step, int anti_bin)
+int GMT_prep_polygons (struct GMT_GSHHS_POL **p_old, int np, BOOLEAN sample, double step, int anti_bin)
 {
 	/* This function will go through each of the polygons and determine
 	 * if the polygon is clipped by the map boundary, and if so if it
@@ -669,14 +708,13 @@ int GMT_prep_polygons (struct GMT_GSHHS_POL **p_old, int np, BOOLEAN greenwich,
 	 * Upon return the polygons are in x,y inches, not degrees.
 	 *
 	 * *p is the array of np polygons
-	 * greenwich is TRUE if area crosses Greenwich
 	 * sample is TRUE if we need to resample the polygons to reduce point spacing
 	 * step is the new maximum point separation in degrees
 	 * anti_bin, if >= 0, indicates a possible problem bin at the antipole using -JE only
 	 * We also explicitly close all polygons if they are not so already.
 	 */
 
-	int k, np_new, n_use, n, start, n_alloc;
+	GMT_LONG k, np_new, n_use, n, start, n_alloc;
 	BOOLEAN close;
 	double *xtmp, *ytmp;
 	struct GMT_GSHHS_POL *p;
@@ -687,12 +725,12 @@ int GMT_prep_polygons (struct GMT_GSHHS_POL **p_old, int np, BOOLEAN greenwich,
 		
 	for (k = 0; k < np; k++) {
 			
-		if (sample) p[k].n = GMT_fix_up_path (&p[k].lon, &p[k].lat, p[k].n, greenwich, step);
+		if (sample) p[k].n = GMT_fix_up_path (&p[k].lon, &p[k].lat, (GMT_LONG)p[k].n, step, 0);
 		
 		/* Clip polygon against map boundary if necessary and return plot x,y in inches */
 				
-		if ((n = GMT_clip_to_map (p[k].lon, p[k].lat, p[k].n, &xtmp, &ytmp)) == 0) {	/* Completely outside */
-			p[k].n = 0;
+		if ((n = GMT_clip_to_map (p[k].lon, p[k].lat, (GMT_LONG)p[k].n, &xtmp, &ytmp)) == 0) {	/* Completely outside */
+			p[k].n = 0;	/* Note the memory in lon, lat not freed yet */
 			continue;
 		}
 			
@@ -947,107 +985,41 @@ void shore_prepare_sides (struct GMT_SHORE *c, int dir)
 	}
 }
 
-BOOLEAN shore_getpathname (char *name, char *path) {
+char *GMT_shore_getpathname (char *stem, char *path) {
 	/* Prepends the appropriate directory to the file name
-	 * and returns TRUE if file is readable. */
+	 * and returns path if file is readable, NULL otherwise */
 	 
-	BOOLEAN found = FALSE;
-	BOOLEAN shore_conffile (char *name, char *dir, char *path);
+	FILE *fp = NULL;
 	char dir[BUFSIZ];
 
 	/* This is the order of checking:
-	 * 1. Is there a GMT_USERDIR with a coastline.conf in it? If so use its information
-	 * 2. Look in GMTHOME/share/coast
-	 * 3. Look in GMTHOME/share (backward check)
-	 * 4. Look for GMTHOME/share/coastline.conf and use its information
-	 * 5. Give up
+	 * 1. Is there a file coastline.conf in current directory, GMT_USERDIR or GMT_SHAREDIR[/coast]?
+	 *    If so, use its information
+	 * 2. Look in current directory, GMT_USERDIR or GMT_SHAREDIR[/coast] for file "name".
 	 */
 	 
-	/* 1. First check the $GMT_USERDIR directory */
+	/* 1. First check for coastline.conf */
 	
-	if (GMT_USERDIR) {		/* See if we have a GMT_USERDIR with a coastline.conf */
-		sprintf (dir, "%s%ccoastline.conf", GMT_USERDIR, DIR_DELIM);
-		found = shore_conffile (name, dir, path);
-		if (found) return (TRUE);
-	}
-	
-	/* 2. Then check the $GMTHOME/share/coast directory */
+	if (GMT_getsharepath ("conf", "coastline", ".conf", path) || GMT_getsharepath ("coast", "coastline", ".conf", path)) {
 
-	sprintf (path, "%s%cshare%ccoast%c%s", GMTHOME, DIR_DELIM, DIR_DELIM, DIR_DELIM, name);
-	if (!access (path, R_OK)) return (TRUE);	/* File exists and is readable, return with name */
+		/* We get here if coastline.conf exists - search among its directories for the named file */
 
-	/* File was not readable.  Now check if it exists */
-
-	if (!access (path, F_OK))  { /* Kicks in if file is there, meaning it has the wrong permissions */
-		fprintf (stderr, "%s: Error: GMT does not have permission to open %s!\n", GMT_program, path);
-		exit (EXIT_FAILURE);
-	}
-	
-	/* 3. Nothing in share/coast; do a backwards-compatible check in the $GMTHOME/share directory */
-
-	sprintf (path, "%s%cshare%c%s", GMTHOME, DIR_DELIM, DIR_DELIM, name);
-	if (!access (path, R_OK)) return (TRUE);	/* File exists and is readable, return with name */
-
-	/* File was not readable.  Now check if it exists */
-
-	if (!access (path, F_OK))  { /* Kicks in if file is there, meaning it has the wrong permissions */
-		fprintf (stderr, "%s: Error: GMT does not have permission to open %s!\n", GMT_program, path);
-		exit (EXIT_FAILURE);
-	}
-
-	/* 4. File is not there.  Thus, we check if a coastline.conf file exists
-	 * It is not an error if we cannot find the named file, only if it is found
-	 * but cannot be read due to permission problems */
-
-	sprintf (dir, "%s%cshare%ccoastline.conf", GMTHOME, DIR_DELIM, DIR_DELIM);
-	found = shore_conffile (name, dir, path);
-	
-	return (found);
-}
-
-BOOLEAN shore_conffile (char *name, char *dir, char *path) {
-	BOOLEAN found = FALSE;
-	FILE *fp;
-	
-	/* Given the dir of a coastline.conf file, look to see if it can be found/read,
-	 * and if so return the directory given.
-	 */
-	 
-	if (!access (dir, F_OK))  { /* File exists... */
-		if (access (dir, R_OK)) {	/* ...but cannot be read */
-			fprintf (stderr, "%s: Error: GMT does not have permission to open %s!\n", GMT_program, dir);
-			exit (EXIT_FAILURE);
-		}
-	}
-	else {	/* There is no coastline.conf file to use; we're out of luck */
-		return (FALSE);
-	}
-
-	/* We get here if coastline.conf exists - search among its directories for the named file */
-
-	if ((fp = fopen (dir, "r")) == NULL) {	/* This shouldn't be necessary, but cannot hurt */
-		fprintf (stderr, "%s: Error: Cannot open configuration file %s\n", GMT_program, dir);
-		exit (EXIT_FAILURE);
-	}
-
-	found = FALSE;
-	while (!found && fgets (dir, BUFSIZ, fp)) {	/* Loop over all input lines until found or done */
-		if (dir[0] == '#' || dir[0] == '\n') continue;	/* Comment or blank */
-
-		GMT_chop (dir);		/* Chop off LF or CR/LF */
-		sprintf (path, "%s%c%s", dir, DIR_DELIM, name);
-		if (!access (path, F_OK)) {	/* TRUE if file exists */
-			if (!access (path, R_OK)) {	/* TRUE if file is readable */
-				found = TRUE;
-			}
-			else {
-				fprintf (stderr, "%s: Error: GMT does not have permission to open %s!\n", GMT_program, path);
-				exit (EXIT_FAILURE);
+		fp = fopen (path, "r");
+		while (fgets (dir, BUFSIZ, fp)) {	/* Loop over all input lines until found or done */
+			if (dir[0] == '#' || dir[0] == '\n') continue;	/* Comment or blank */
+			GMT_chop (dir);		/* Chop off LF or CR/LF */
+			sprintf (path, "%s%c%s%s", dir, DIR_DELIM, stem, ".cdf");
+			if (!access (path, R_OK)) {
+				fclose (fp);
+				return (path);
 			}
 		}
+		fclose (fp);
 	}
-
-	fclose (fp);
 	
-	return (found);
+	/* 2. Then check for the named file itself */
+
+	if (GMT_getsharepath ("coast", stem, ".cdf", path)) return (path);
+
+	return (NULL);
 }
