@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: psmask.c,v 1.44 2006/10/11 02:25:13 remko Exp $
+ *	$Id: psmask.c,v 1.71 2009/01/12 15:51:58 guru Exp $
  *
- *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,7 @@
  *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
- * psmask tries to achieve masking using one of two different approachs:
+ * psmask tries to achieve masking using one of two different approaches:
  * The default way of operation is: Instead of painting tiles where there's no
  * data [or where there is data] psmask uses contouring to find the polygons
  * that contain the data [or contain the regions with no data].  For many types
@@ -40,8 +40,6 @@
 #include "gmt.h"
 #include "pslib.h"
 
-#define PSMASK_IJ(i,j) ((i) + (j) * h->nx)
-
 struct PSMASK_CTRL {
 	struct C {	/* -C */
 		BOOLEAN active;
@@ -49,6 +47,7 @@ struct PSMASK_CTRL {
 	struct D {	/* -D<dumpfile> */
 		BOOLEAN active;
 		char *file;
+		int min_pts;
 	} D;
 	struct E {	/* -Eazim/elev */
 		BOOLEAN active;
@@ -86,12 +85,15 @@ struct PSMASK_INFO {
 
 int main (int argc, char **argv)
 {
-	int i, j, ij, n, nm, n_edges, di, dj, ii, jj, n_expected_fields, *edge;
-	int section, n_alloc, n_read, n_fields, one_or_zero, distance_flag = 0;
+	GMT_LONG ij, n, nm;
+	
+	int i, j, n_edges, di, dj, ii, jj, n_expected_fields, *edge;
+	int section, n_fields, distance_flag = 0, n_plus = 0, k;
 
-	BOOLEAN error = FALSE, first = TRUE, use_poly, node_only;
+	GMT_LONG n_read, n_alloc;
+	BOOLEAN error = FALSE, first = TRUE, node_only;
 
-	char dfile[BUFSIZ], line[BUFSIZ], *grd;
+	char line[BUFSIZ], *grd;
 
 	double *in, distance, x0, y0, x1, y1, shrink = 1.0;
 	double xinc2, yinc2, *x, *y;
@@ -102,10 +104,13 @@ int main (int argc, char **argv)
 	struct PSMASK_INFO info;
 	struct PSMASK_CTRL *Ctrl;
 
-	void draw_clip_contours (double *xx, double *yy, int nn, int rgb[], int id, int flag);
-	void dump_clip_contours (struct PSMASK_INFO *info, double *xx, double *yy, int nn, int id, char *file);
-	void shrink_clip_contours (double *x, double *y, int n, double w, double e);
-	int clip_contours (struct PSMASK_INFO *info, char *grd, struct GRD_HEADER *h, double xinc2, double yinc2, int *edge, int first, double **x, double **y, int *max);
+#ifdef DEBUG
+	BOOLEAN debug = FALSE;
+#endif
+	void draw_clip_contours (double *xx, double *yy, GMT_LONG nn, int rgb[], int id, int flag);
+	void dump_clip_contours (struct PSMASK_INFO *info, double *xx, double *yy, GMT_LONG nn, int id, char *file);
+	void shrink_clip_contours (double *x, double *y, GMT_LONG n, double w, double e);
+	int clip_contours (struct PSMASK_INFO *info, char *grd, struct GRD_HEADER *h, double xinc2, double yinc2, int *edge, int first, double **x, double **y, GMT_LONG *max);
 	void *New_Psmask_Ctrl (), Free_Psmask_Ctrl (struct PSMASK_CTRL *C);
 	
 	argc = GMT_begin (argc, argv);
@@ -120,6 +125,8 @@ int main (int argc, char **argv)
 	GMT_init_fill (&Ctrl->G.fill, -1, -1, -1);
 	xinc2 = yinc2 = 0.0;
 
+	Ctrl->D.min_pts = 0;
+
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch (argv[i][1]) {
@@ -152,12 +159,23 @@ int main (int argc, char **argv)
 					break;
 				case 'D':	/* Dump the polygons to files */
 					Ctrl->D.active = TRUE;
-					GMT_free ((void *)Ctrl->D.file);
+					free ((void *)Ctrl->D.file);
+
+					for (n_plus = 0, k = 2; argv[i][k]; k++) {
+						if (argv[i][k] == '+' && argv[i][k+1] == 'n') {
+							Ctrl->D.min_pts = atoi(&argv[i][k + 2]);
+							n_plus = 1;
+							break;
+						}
+					}
+
+					if (n_plus)	/* If extra option rip it before check if there is a prefix */
+						argv[i][k] = '\0';
 					Ctrl->D.file = strdup (&argv[i][2]);
 					break;
 				case 'E':
 					Ctrl->E.active = TRUE;
-					sscanf (&argv[i][2], "%lf/%lf", &Ctrl->E.azimuth, &Ctrl->E.elevation);
+					error += GMT_get_proj3D (&argv[i][2], &Ctrl->E.azimuth, &Ctrl->E.elevation);
 					break;
 				case 'F':
 					Ctrl->F.active = TRUE;
@@ -187,6 +205,11 @@ int main (int argc, char **argv)
 				case 'T':
 					Ctrl->T.active = TRUE;
 					break;
+#ifdef DEBUG
+				case 'd':
+					debug = TRUE;
+					break;
+#endif
 				default:
 					error = TRUE;
 					GMT_default_error (argv[i][1]);
@@ -202,11 +225,11 @@ int main (int argc, char **argv)
 	}
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
-		fprintf (stderr,"psmask %s - Clipping of 2-D data sets\n\n", GMT_VERSION);
+		fprintf (stderr,"psmask %s - Masking or clipping of 2-D data sets\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: psmask <xyz-file> %s %s %s\n", GMT_I_OPT, GMT_J_OPT, GMT_Rgeo_OPT);
-		fprintf (stderr, "\t[%s] [-C] -D[-]<file> [-Eaz/el] [-F] [-G<fill>] [%s] [-K] [%s]\n", GMT_B_OPT, GMT_H_OPT, GMT_Mo_OPT);
+		fprintf (stderr, "\t[%s] [-C] -D[-]<file>[+n<n_pts>] [%s] [-F] [-G<fill>] [%s] [-K] [%s]\n", GMT_B_OPT, GMT_E_OPT, GMT_H_OPT, GMT_Mo_OPT);
 		fprintf (stderr, "\t[-N] [-O] [-P] [-S<radius>[k]] [-T] [%s] [-V] [%s]\n", GMT_U_OPT, GMT_X_OPT);
-		fprintf (stderr, "\t[%s] [%s] [%s] [%s]\n\n", GMT_Y_OPT, GMT_c_OPT, GMT_t_OPT, GMT_b_OPT);
+		fprintf (stderr, "\t[%s] [%s] [%s] [%s] [-n]\n\n", GMT_Y_OPT, GMT_c_OPT, GMT_t_OPT, GMT_b_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
@@ -220,7 +243,8 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t-D dumps the clip-paths to files using the prefix <file>_ [mask_]\n");
 		fprintf (stderr, "\t   Ignored if -T is specified.  If prefix begins with - we will instead\n");
 		fprintf (stderr, "\t   use the rest of the name for a single multi-segment file.\n");
-		fprintf (stderr, "\t-E set azimuth and elevation of viewpoint for 3-D perspective [180/90]\n");
+		fprintf (stderr, "\t   Append +n<n_pts> to limit the number of points in files to a minimum of n_pts.\n");
+		GMT_explain_option ('E');
 		fprintf (stderr, "\t-F Force pixel registration [Default is gridline registration]\n");
 		GMT_fill_syntax ('G', "Select fill color/pattern [Default is no fill].");
 		GMT_explain_option ('H');
@@ -252,6 +276,8 @@ int main (int argc, char **argv)
 		exit (EXIT_FAILURE);
 	}
 
+	GMT_check_lattice (&Ctrl->I.xinc, &Ctrl->I.yinc, &Ctrl->F.active, &Ctrl->I.active);
+
 	if (!Ctrl->C.active) {
 		if (Ctrl->E.elevation <= 0.0 || Ctrl->E.elevation > 90.0) {
 			fprintf (stderr, "%s: GMT SYNTAX ERROR -E option:  Elevation must be in 0-90 range\n", GMT_program);
@@ -273,8 +299,8 @@ int main (int argc, char **argv)
 			fprintf (stderr, "%s: GMT SYNTAX ERROR -I option:  Must specify positive increments\n", GMT_program);
 			error++;
 		}
-		if (Ctrl->S.active && Ctrl->S.radius <= 0.0) {
-			fprintf (stderr, "%s: GMT SYNTAX ERROR -S option:  Radius must be positive\n", GMT_program);
+		if (Ctrl->S.active && (Ctrl->S.radius <= 0.0 || GMT_is_dnan (Ctrl->S.radius))) {
+			fprintf (stderr, "%s: GMT SYNTAX ERROR.  Radius is NaN or negative\n", GMT_program);
 			error++;
 		}
 		if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] == 0) GMT_io.ncol[GMT_IN] = 2;
@@ -292,13 +318,15 @@ int main (int argc, char **argv)
 		h.xy_off = 0.5 * h.node_offset;
 		xinc2 = 0.5 * h.x_inc;
 		yinc2 = 0.5 * h.y_inc;
-		one_or_zero = 1 - h.node_offset;
 		GMT_RI_prepare (&h);	/* Ensure -R -I consistency and set nx, ny */
 	}
 
 	if (error) exit (EXIT_FAILURE);
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
+	/* Now test also if we must provide a default -J */
+	if (Ctrl->D.active && project_info.projection < 0) {		/* Is this the right way of testing it? */
+		GMT_parse_J_option ("x1");
+	}
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
@@ -308,7 +336,7 @@ int main (int argc, char **argv)
 	if (Ctrl->S.unit == 'K') distance_flag = 2;
 	z_project.view_azimuth = Ctrl->E.azimuth;
 	z_project.view_elevation = Ctrl->E.elevation;
-	if (!Ctrl->C.active) GMT_map_setup (h.x_min, h.x_max, h.y_min, h.y_max);
+	if (!Ctrl->C.active) GMT_err_fail (GMT_map_setup (h.x_min, h.x_max, h.y_min, h.y_max), "");
 
 	if (!project_info.x_off_supplied && GMT_ps.overlay) GMT_ps.x_origin = 0.0;
 	if (!project_info.y_off_supplied && GMT_ps.overlay) GMT_ps.y_origin = 0.0;
@@ -331,14 +359,16 @@ int main (int argc, char **argv)
 		nm = h.nx * h.ny;
 		grd = (char *) GMT_memory (VNULL, (size_t)nm, sizeof (char), GMT_program);
 
+		/* Add GMT_CONV_LIMIT to ensure that special case radius = inc --> irint(0.5) actually rounds to 1 */
+		
 		if (distance_flag) {
 			shrink = cos (0.5 * (h.y_min + h.y_max) * D2R);
-			di = (int)ceil (Ctrl->S.radius / (project_info.KM_PR_DEG * h.x_inc * shrink));
-			dj = (int)ceil (Ctrl->S.radius / (project_info.KM_PR_DEG * h.y_inc));
+			di = (int)ceil (Ctrl->S.radius / (project_info.KM_PR_DEG * h.x_inc * shrink) + GMT_CONV_LIMIT);
+			dj = (int)ceil (Ctrl->S.radius / (project_info.KM_PR_DEG * h.y_inc) + GMT_CONV_LIMIT);
 		}
 		else {
-			di = irint (0.5 * Ctrl->S.radius / h.x_inc);
-			dj = irint (0.5 * Ctrl->S.radius / h.y_inc);
+			di = irint (0.5 * Ctrl->S.radius / h.x_inc + GMT_CONV_LIMIT);
+			dj = irint (0.5 * Ctrl->S.radius / h.y_inc + GMT_CONV_LIMIT);
 		}
 		if (distance_flag == 2 && !GMT_IS_SPHERICAL) distance_flag = 3;	/* Use geodesics */
 		switch (distance_flag) {	/* Take different action depending on how we want distances calculated */
@@ -357,7 +387,9 @@ int main (int argc, char **argv)
 			break;
 		}
 		node_only = (di == 0 && dj == 0);
-
+		if (node_only && Ctrl->S.radius > 0.0) {
+			fprintf (stderr, "%s: Warning: Your search radius is too small to have any effect and is ignored.\n", GMT_program);
+		}
 		if (fp == NULL) {
 			fp = GMT_stdin;
 #ifdef SET_IO_MODE
@@ -377,7 +409,7 @@ int main (int argc, char **argv)
 			if ((GMT_io.status & GMT_IO_EOF)) continue;	/* At EOF */
 
 			if (GMT_io.status & GMT_IO_MISMATCH) {
-				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected_fields, n_read);
+				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n_read);
 				exit (EXIT_FAILURE);
 			}
 			n_read++;
@@ -416,7 +448,7 @@ int main (int argc, char **argv)
 		}
 
 		if (fp != GMT_stdin) GMT_fclose (fp);
-		if (gmtdefs.verbose) fprintf (stderr, "%s: Read %d data points\n", GMT_program, n_read);
+		if (gmtdefs.verbose) fprintf (stderr, "%s: Read %ld data points\n", GMT_program, n_read);
 
 		if (Ctrl->N.active) for (i = 0; i < nm; i++) grd[i] = 1 - grd[i];	/* Reverse sense of test */
 
@@ -425,6 +457,15 @@ int main (int argc, char **argv)
 		for (i = 0, ij = (h.ny-1) * h.nx; i < h.nx; i++) grd[i] = grd[i+ij] = FALSE;
 		for (j = 0; j < h.ny; j++) grd[j*h.nx] = grd[(j+1)*h.nx-1] = FALSE;
 
+#ifdef DEBUG
+		if (debug) {
+			float *z;
+			z = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
+			for (i = 0; i < nm; i++) z[i] = (float)grd[i];
+			GMT_write_grd ("psmask.grd", &h, z, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE);
+			GMT_free ((void *)z);
+		}
+#endif
 		if (!Ctrl->T.active) {	/* Must trace the outline of ON/OFF values in grd */
 			/* Arrays holding the contour xy values */
 			x = (double *) GMT_memory (VNULL, (size_t)GMT_CHUNK, sizeof (double), GMT_program);
@@ -442,32 +483,31 @@ int main (int argc, char **argv)
 			first = TRUE;
 			while ((n = clip_contours (&info, grd, &h, xinc2, yinc2, edge, first, &x, &y, &n_alloc)) > 0) {
 				shrink_clip_contours (x, y, n, h.x_min, h.x_max);
-				if (Ctrl->D.active) dump_clip_contours (&info, x, y, n, section, dfile);
+				if (Ctrl->D.active && n > Ctrl->D.min_pts) dump_clip_contours (&info, x, y, n, section, Ctrl->D.file);
 				draw_clip_contours (x, y, n, Ctrl->G.fill.rgb, section, first);
 				first = FALSE;
 				section++;
 			}
 
-			draw_clip_contours (x, y, 0, Ctrl->G.fill.rgb, section, 2);	/* Activate clip-path */
+			draw_clip_contours (x, y, (GMT_LONG)0, Ctrl->G.fill.rgb, section, 2);	/* Activate clip-path */
 
 			GMT_free ((void *)edge);
 			GMT_free ((void *)x);
 			GMT_free ((void *)y);
 		}
 		else {	/* Just paint tiles */
-			int start, plot_n, n_use, np;
+			int start;
+			GMT_LONG n_use, np, plot_n;
 			double *grd_x, grd_y, y_bot, y_top, *xx, *yy, *xp, *yp;
 			if (gmtdefs.verbose) fprintf (stderr, "%s: Tiling...\n", GMT_program);
 			grd_x = (double *) GMT_memory (VNULL, (size_t)h.nx, sizeof (double), GMT_program);
 			for (i = 0; i < h.nx; i++) grd_x[i] = GMT_i_to_x (i, h.x_min, h.x_max, h.x_inc, h.xy_off, h.nx);
 
-			use_poly = (Ctrl->G.fill.use_pattern || project_info.three_D);
-
 			for (j = 0; j < h.ny; j++) {
 				grd_y = GMT_j_to_y (j, h.y_min, h.y_max, h.y_inc, h.xy_off, h.ny);
 				y_bot = grd_y - yinc2;
 				y_top = grd_y + yinc2;
-				ij = j * h.nx + 1;
+				ij = GMT_IJ (j, 1, h.nx);
 				for (i = 0; i < h.nx; i++, ij++) {
 					if (((int)grd[ij]) == 0) continue;
 
@@ -526,9 +566,9 @@ int main (int argc, char **argv)
 	exit (EXIT_SUCCESS);
 }
 
-void draw_clip_contours (double *xx, double *yy, int nn, int rgb[], int id, int flag)
+void draw_clip_contours (double *xx, double *yy, GMT_LONG nn, int rgb[], int id, int flag)
 {
-	int i;
+	GMT_LONG i;
 	double x, y;
 	char comment[GMT_TEXT_LEN];
 
@@ -553,14 +593,15 @@ void draw_clip_contours (double *xx, double *yy, int nn, int rgb[], int id, int
 	}
 }
 
-void dump_clip_contours (struct PSMASK_INFO *info, double *xx, double *yy, int nn, int id, char *file)
+void dump_clip_contours (struct PSMASK_INFO *info, double *xx, double *yy, GMT_LONG nn, int id, char *file)
 {
-	int i;
+	GMT_LONG i;
 	double out[2];
 	char fname[BUFSIZ];
 	FILE *fp;
 
 	if (nn < 2) return;
+	if (nn == 2 && GMT_IS_ZERO (xx[1] - xx[0]) && GMT_IS_ZERO (yy[1] - yy[0])) return;
 
 	if (file[0] == '-') {	/* Want a single multi-segment file */
 		sprintf (fname, "%s", &file[1]);
@@ -573,6 +614,7 @@ void dump_clip_contours (struct PSMASK_INFO *info, double *xx, double *yy, int n
 			exit (EXIT_FAILURE);
 		}
 		info->first_dump = FALSE;
+		GMT_write_segmentheader (fp, 2);
 	}
 	else {
 		if (GMT_io.binary[GMT_OUT])
@@ -591,17 +633,18 @@ void dump_clip_contours (struct PSMASK_INFO *info, double *xx, double *yy, int n
 	GMT_fclose (fp);
 }
 
-int clip_contours (struct PSMASK_INFO *info, char *grd, struct GRD_HEADER *h, double xinc2, double yinc2, int *edge, int first, double **x, double **y, int *max)
+int clip_contours (struct PSMASK_INFO *info, char *grd, struct GRD_HEADER *h, double xinc2, double yinc2, int *edge, int first, double **x, double **y, GMT_LONG *max)
 {
 	/* The routine finds the zero-contour in the grd dataset.  it assumes that
 	 * no node has a value exactly == 0.0.  If more than max points are found
 	 * trace_clip_contours will try to allocate more memory in blocks of GMT_CHUNK points
 	 */
 	 
-	static int i0, j0;
-	int i, j, ij, n = 0, n_edges, edge_word, edge_bit;
+	static int i0, j0, side;
+	GMT_LONG ij;
+	int i, j, n = 0, n_edges, edge_word, edge_bit;
 	BOOLEAN go_on = TRUE;
-	int trace_clip_contours (struct PSMASK_INFO *info, char *grd, int *edge, struct GRD_HEADER *h, double xinc2, double yinc2, double **xx, double **yy, int i, int j, int kk, int *max);
+	int trace_clip_contours (struct PSMASK_INFO *info, char *grd, int *edge, struct GRD_HEADER *h, double xinc2, double yinc2, double **xx, double **yy, int i, int j, int kk, GMT_LONG *max);
 	 
 	 
 	n_edges = h->ny * (int) ceil (h->nx / 16.0);
@@ -611,6 +654,7 @@ int clip_contours (struct PSMASK_INFO *info, char *grd, struct GRD_HEADER *h, do
 		info->offset = n_edges / 2;
 	 	i0 = 0;	/* Begin with upper left bin which is i = 0 and j = 1 */
 	 	j0 = 1;
+		side = 4;	/* Vertical interior gridlines */
 		info->p[0] = info->p[4] = 0;	info->p[1] = 1;	info->p[2] = 1 - h->nx;	info->p[3] = -h->nx;
 		info->i_off[0] = info->i_off[2] = info->i_off[3] = info->i_off[4] = 0;	info->i_off[1] =  1;
 		info->j_off[0] = info->j_off[1] = info->j_off[3] = info->j_off[4] = 0;	info->j_off[2] = -1;
@@ -620,38 +664,65 @@ int clip_contours (struct PSMASK_INFO *info, char *grd, struct GRD_HEADER *h, do
 
 	/* Loop over interior boxes */
 
-	for (j = j0; go_on && j < h->ny; j++) {
-		ij = i0 + j * h->nx;
-		for (i = i0; go_on && i < h->nx-1; i++, ij++) {	/* nx-1 since the last bin starts at nx-2 and ends at nx-1 */
-			edge_word = ij / 32 + info->offset;
-			edge_bit = ij % 32;
-			if (!(edge[edge_word] & info->bit[edge_bit]) && ((grd[ij]+grd[ij-h->nx]) == 1)) { /* Start tracing contour */
-				*x[0] = GMT_i_to_x (i, h->x_min, h->x_max, h->x_inc, h->xy_off, h->nx);
-				*y[0] = GMT_j_to_y (j, h->y_min, h->y_max, h->y_inc, h->xy_off, h->ny);
-				edge[edge_word] |= info->bit[edge_bit];
-				n = trace_clip_contours (info, grd, edge, h, xinc2, yinc2, x, y, i, j, 3, max);
-				go_on = FALSE;
-				i0 = i + 1;
-				j0 = j;	/* Return to finish this row later */
+	if (side == 4) {
+		for (j = j0; go_on && j < h->ny; j++) {
+			ij = GMT_IJ (j, i0, h->nx);
+			for (i = i0; go_on && i < h->nx-1; i++, ij++) {	/* nx-1 since the last bin starts at nx-2 and ends at nx-1 */
+				edge_word = ij / 32 + info->offset;
+				edge_bit = ij % 32;
+				if (!(edge[edge_word] & info->bit[edge_bit]) && ((grd[ij]+grd[ij-h->nx]) == 1)) { /* Start tracing contour */
+					*x[0] = GMT_i_to_x (i, h->x_min, h->x_max, h->x_inc, h->xy_off, h->nx);
+					*y[0] = GMT_j_to_y (j, h->y_min, h->y_max, h->y_inc, h->xy_off, h->ny);
+					edge[edge_word] |= info->bit[edge_bit];
+					n = trace_clip_contours (info, grd, edge, h, xinc2, yinc2, x, y, i, j, 3, max);
+					go_on = FALSE;
+					i0 = i + 1;
+					j0 = j;	/* Return to finish this row later */
+				}
+			}
+			if (go_on) i0 = 0;	/* Go to start of next row unless we found something */
+		}
+		if (n == 0) {
+			side = 5;
+			i0 = 0;
+			j0 = 1;
+		}
+	}
+	if (n == 0 && side == 5) {
+		for (j = j0; go_on && j < h->ny; j++) {
+			ij = GMT_IJ (j, i0, h->nx);
+			for (i = i0; go_on && i < h->nx-1; i++, ij++) {
+				edge_word = ij / 32 + info->offset;
+				edge_bit = ij % 32;
+				if (!(edge[edge_word] & info->bit[edge_bit]) && ((grd[ij]+grd[ij+1]) == 1)) { /* Start tracing contour */
+					*x[0] = GMT_i_to_x (i, h->x_min, h->x_max, h->x_inc, h->xy_off, h->nx);
+					*y[0] = GMT_j_to_y (j, h->y_min, h->y_max, h->y_inc, h->xy_off, h->ny);
+					edge[edge_word] |= info->bit[edge_bit];
+					n = trace_clip_contours (info, grd, edge, h, xinc2, yinc2, x, y, i, j, 2, max);
+					go_on = FALSE;
+					i0 = i + 1;
+					j0 = j;	/* Return to finish this row later */
+				}
+				if (go_on) i0 = 1;
 			}
 		}
-		if (go_on) i0 = 0;	/* Go to start of next row unless we found something */
-	}
+	}	
 
 	return (n);
 }
 
-int trace_clip_contours (struct PSMASK_INFO *info, char *grd, int *edge, struct GRD_HEADER *h, double xinc2, double yinc2, double **xx, double **yy, int i, int j, int kk, int *max)
+int trace_clip_contours (struct PSMASK_INFO *info, char *grd, int *edge, struct GRD_HEADER *h, double xinc2, double yinc2, double **xx, double **yy, int i, int j, int kk, GMT_LONG *max)
 {
-	int n = 1, k, k0, ij, ij0, n_cuts, kk_opposite, first_k, more;
+	int n = 1, k, k0, n_cuts, kk_opposite, first_k, more;
 	int edge_word, edge_bit, m;
+	GMT_LONG ij, ij0;
 	double xk[4], yk[4], x0, y0;
 
 	m = *max - 2;
 	
 	more = TRUE;
 	do {
-		ij = i + j * h->nx;
+		ij = GMT_IJ (j, i, h->nx);
 		x0 = GMT_i_to_x (i, h->x_min, h->x_max, h->x_inc, h->xy_off, h->nx);
 		y0 = GMT_j_to_y (j, h->y_min, h->y_max, h->y_inc, h->xy_off, h->ny);
 		n_cuts = 0;
@@ -665,7 +736,7 @@ int trace_clip_contours (struct PSMASK_INFO *info, char *grd, int *edge, struct
 
 			/* Skip edge already has been used */
 
-			ij0 = PSMASK_IJ (i + info->i_off[k], j + info->j_off[k]);
+			ij0 = GMT_IJ (j + info->j_off[k], i + info->i_off[k], h->nx);
 			edge_word = ij0 / 32 + info->k_off[k] * info->offset;
 			edge_bit = ij0 % 32;
 			if (edge[edge_word] & info->bit[edge_bit]) continue;
@@ -701,16 +772,19 @@ int trace_clip_contours (struct PSMASK_INFO *info, char *grd, int *edge, struct
 		}
 
 		if (n > m) {	/* Must try to allocate more memory */
-			*max += GMT_CHUNK;
-			m += GMT_CHUNK;
+			*max = (*max == 0) ? GMT_CHUNK : ((*max) << 1);
+			m = (m == 0) ? GMT_CHUNK : (m << 1);
 			*xx = (double *) GMT_memory ((void *)*xx, (size_t)(*max), sizeof (double), "trace_clip_contours");
 			*yy = (double *) GMT_memory ((void *)*yy, (size_t)(*max), sizeof (double), "trace_clip_contours");
 		}
 		if (n_cuts == 0) {	/* Close interior contour and return */
-			if (fmod ((*xx[0] - xinc2), h->x_inc) == 0.0)	/* On side 1 or 3 */
-				first_k = ((*xx[0] - x0) == 0.0) ? 3 : 1;
+			/* if (fmod ((*xx[0] - xinc2), h->x_inc) == 0.0) */	/* On side 1 or 3 */
+			if (GMT_IS_ZERO (fmod ((*xx[0] - xinc2), h->x_inc)))	/* On side 1 or 3 */
+				/* first_k = ((*xx[0] - x0) == 0.0) ? 3 : 1; */
+				first_k = GMT_IS_ZERO (*xx[0] - x0) ? 3 : 1;
 			else 	/* On side 0 or 2 */
-				first_k = ((*yy[0] - y0) == 0.0) ? 0 : 2;
+				/* first_k = ((*yy[0] - y0) == 0.0) ? 0 : 2; */
+				first_k = GMT_IS_ZERO (*yy[0] - y0) ? 0 : 2;
 			kk_opposite = (first_k + 2) % 4;
 			if (k0 != kk_opposite) {
 				(*xx)[n] = x0 + 0.5*h->x_inc;
@@ -745,7 +819,7 @@ int trace_clip_contours (struct PSMASK_INFO *info, char *grd, int *edge, struct
 			n++;
 		}
 		if (more) {	/* Mark this edge as used */
-			ij0 = PSMASK_IJ (i + info->i_off[kk], j + info->j_off[kk]);
+			ij0 = GMT_IJ (j + info->j_off[kk], i + info->i_off[kk], h->nx);
 			edge_word = ij0 / 32 + info->k_off[kk] * info->offset;
 			edge_bit = ij0 % 32;
 			edge[edge_word] |= info->bit[edge_bit];
@@ -761,10 +835,10 @@ int trace_clip_contours (struct PSMASK_INFO *info, char *grd, int *edge, struct
 	return (n);
 }
 
-void shrink_clip_contours (double *x, double *y, int n, double w, double e)
+void shrink_clip_contours (double *x, double *y, GMT_LONG n, double w, double e)
 {
 	/* Moves outside points to boundary */
-	int i;
+	GMT_LONG i;
 
 	for (i = 0; i < n; i++) {
 		if (x[i] < w)
@@ -781,7 +855,7 @@ void shrink_clip_contours (double *x, double *y, int n, double w, double e)
 void *New_Psmask_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct PSMASK_CTRL *C;
 	
-	C = (struct PSMASK_CTRL *) GMT_memory (VNULL, 1, sizeof (struct PSMASK_CTRL), "New_Psmask_Ctrl");
+	C = (struct PSMASK_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PSMASK_CTRL), "New_Psmask_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 		
@@ -792,5 +866,6 @@ void *New_Psmask_Ctrl () {	/* Allocate and initialize a new control structure */
 }
 
 void Free_Psmask_Ctrl (struct PSMASK_CTRL *C) {	/* Deallocate control structure */
+	if (C->D.file) free ((void *)C->D.file);	
 	GMT_free ((void *)C);	
 }
