@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: trend1d.c,v 1.29 2006/10/27 23:45:23 pwessel Exp $
+ *	$Id: trend1d.c,v 1.40 2009/01/09 04:02:34 guru Exp $
  *
- *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -144,15 +144,16 @@ struct	TREND1D_DATA {
 	double	w;
 };
 
-int main(int argc, char **argv)
+int main (int argc, char **argv)
 {
-	int i, j, k, n_data, n_outputs, n_model, np, significant, rank, n_req;
-
+	int i, j, k,n_outputs, n_model, significant, rank, n_req;
+	
+	GMT_LONG  n_data, np;
 	BOOLEAN	error = FALSE, weighted_output = FALSE;
 
 	double	*gtg, *v, *gtd, *lambda, *workb, *workz, *c_model, *o_model, *w_model, *work;	/* Arrays  */
 	double	xmin, xmax, c_chisq, o_chisq, w_chisq, scale = 1.0, prob;
-	double	get_chisq(struct TREND1D_DATA *data, int n_data, int n_model);
+	double	get_chisq(struct TREND1D_DATA *data, GMT_LONG n_data, int n_model);
 
 	char	format[BUFSIZ];
 
@@ -161,16 +162,16 @@ int main(int argc, char **argv)
 	struct	TREND1D_DATA *data;
 	struct TREND1D_CTRL *Ctrl;
 
-	void read_data(struct TREND1D_DATA **data, int *n_data, double *xmin, double *xmax, int weighted_input, double **work, FILE *fp);
-	void write_output(struct TREND1D_DATA *data, int n_data, char *output_choice, int n_outputs), transform_x(struct TREND1D_DATA *data, int n_data, int model_type, double xmin, double xmax);
-	void untransform_x(struct TREND1D_DATA *data, int n_data, int model_type, double xmin, double xmax);
-	void recompute_weights(struct TREND1D_DATA *data, int n_data, double *work, double *scale);
-	void allocate_array_space(int np, double **gtg, double **v, double **gtd, double **lambda, double **workb, double **workz, double **c_model, double **o_model, double **w_model);
+	void read_data(struct TREND1D_DATA **data, GMT_LONG *n_data, double *xmin, double *xmax, int weighted_input, double **work, FILE *fp);
+	void write_output(struct TREND1D_DATA *data, GMT_LONG n_data, char *output_choice, int n_outputs), transform_x(struct TREND1D_DATA *data, GMT_LONG n_data, int model_type, double xmin, double xmax);
+	void untransform_x(struct TREND1D_DATA *data, GMT_LONG n_data, int model_type, double xmin, double xmax);
+	void recompute_weights(struct TREND1D_DATA *data, GMT_LONG n_data, double *work, double *scale);
+	void allocate_array_space(GMT_LONG np, double **gtg, double **v, double **gtd, double **lambda, double **workb, double **workz, double **c_model, double **o_model, double **w_model);
 	void free_the_memory(double *gtg, double *v, double *gtd, double *lambda, double *workb, double *workz, double *c_model, double *o_model, double *w_model, struct TREND1D_DATA *data, double *work);
-	void calc_m_and_r(struct TREND1D_DATA *data, int n_data, double *model, int n_model, int m_type, double *grow);
+	void calc_m_and_r(struct TREND1D_DATA *data, GMT_LONG n_data, double *model, int n_model, int m_type, double *grow);
 	void move_model_a_to_b(double *model_a, double *model_b, int n_model, double *chisq_a, double *chisq_b);
-	void load_gtg_and_gtd(struct TREND1D_DATA *data, int n_data, double *gtg, double *gtd, double *grow, int n_model, int mp, int m_type);
-	void solve_system(double *gtg, double *gtd, double *model, int n_model, int mp, double *lambda, double *v, double *b, double *z, double c_no, int *ir);
+	void load_gtg_and_gtd(struct TREND1D_DATA *data, GMT_LONG n_data, double *gtg, double *gtd, double *grow, int n_model, GMT_LONG mp, int m_type);
+	void solve_system(double *gtg, double *gtd, double *model, int n_model, GMT_LONG mp, double *lambda, double *v, double *b, double *z, double c_no, int *ir);
 	void GMT_cheb_to_pol (double c[], int n, double a, double b);
 	void *New_Trend1d_Ctrl (), Free_Trend1d_Ctrl (struct TREND1D_CTRL *C);
 	
@@ -321,8 +322,6 @@ int main(int argc, char **argv)
 
 	if (error) exit (EXIT_FAILURE);
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
-
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
 		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
@@ -350,12 +349,12 @@ int main(int argc, char **argv)
 	transform_x(data, n_data, Ctrl->N.mode, xmin, xmax);	/* Set domain to [-1, 1] or [-pi, pi]  */
 
 	if (gmtdefs.verbose) {
-		sprintf(format,"%%s:  Read %%d data with X values from %s to %s\n", gmtdefs.d_format, gmtdefs.d_format);
+		sprintf(format,"%%s:  Read %%ld data with X values from %s to %s\n", gmtdefs.d_format, gmtdefs.d_format);
 		fprintf(stderr, format, GMT_program, n_data, xmin, xmax);
 		fprintf(stderr,"N_model\tRank\tChi_Squared\tSignificance\n");
 	}
 
-	sprintf (format, "%%d\t%%d\t%s\t%s\n", gmtdefs.d_format, gmtdefs.d_format);
+	sprintf (format, "%%ld\t%%ld\t%s\t%s\n", gmtdefs.d_format, gmtdefs.d_format);
 
 	if (Ctrl->I.active) {
 		n_model = 1;
@@ -455,7 +454,7 @@ int main(int argc, char **argv)
 	}
 
 	if (gmtdefs.verbose) {
-		sprintf (format, "%%s: Final model stats:  N model parameters %%d.  Rank %%d.  Chi-Squared:  %s\n", gmtdefs.d_format);
+		sprintf (format, "%%s: Final model stats:  N model parameters %%ld.  Rank %%ld.  Chi-Squared:  %s\n", gmtdefs.d_format);
 		fprintf(stderr, format, GMT_program, n_model, rank, c_chisq);
 		fprintf(stderr,"%s: Model Coefficients  (Chebyshev):", GMT_program);
 		sprintf (format, "\t%s", gmtdefs.d_format);
@@ -480,9 +479,10 @@ int main(int argc, char **argv)
 	exit (EXIT_SUCCESS);
 }
 
-void read_data (struct TREND1D_DATA **data, int *n_data, double *xmin, double *xmax, int weighted_input, double **work, FILE *fp)
+void read_data (struct TREND1D_DATA **data, GMT_LONG *n_data, double *xmin, double *xmax, int weighted_input, double **work, FILE *fp)
 {
-	int	i, n_alloc = GMT_CHUNK, n_expected_fields, n_fields;
+	int	n_alloc = GMT_CHUNK, n_expected_fields, n_fields;
+	GMT_LONG i;
 	double	*in;
 	char	buffer[BUFSIZ];
 
@@ -501,7 +501,7 @@ void read_data (struct TREND1D_DATA **data, int *n_data, double *xmin, double *x
 	while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {
 
 		if (GMT_io.status & GMT_IO_MISMATCH) {
-			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected_fields, i);
+			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, i);
 			exit (EXIT_FAILURE);
 		}
 
@@ -520,9 +520,14 @@ void read_data (struct TREND1D_DATA **data, int *n_data, double *xmin, double *x
 		i++;
 
 		if (i == n_alloc) {
-			n_alloc += GMT_CHUNK;
+			n_alloc <<= 1;
 			*data = (struct TREND1D_DATA *) GMT_memory ((void *)*data, (size_t)n_alloc, sizeof(struct TREND1D_DATA), GMT_program);
 		}
+		if (i == INT_MAX) {
+			fprintf (stderr, "%s: ERROR: Cannot process more than %d data points\n", GMT_program, INT_MAX);
+			GMT_free ((void *)data);
+			exit (EXIT_FAILURE);
+		}
 	}
 	if (fp != GMT_stdin) GMT_fclose(fp);
 
@@ -532,7 +537,7 @@ void read_data (struct TREND1D_DATA **data, int *n_data, double *xmin, double *x
 	*n_data = i;
 }
 
-void allocate_array_space (int np, double **gtg, double **v, double **gtd, double **lambda, double **workb, double **workz, double **c_model, double **o_model, double **w_model)
+void allocate_array_space (GMT_LONG np, double **gtg, double **v, double **gtd, double **lambda, double **workb, double **workz, double **c_model, double **o_model, double **w_model)
 {
 	*gtg = (double *) GMT_memory (VNULL, (size_t)(np*np), sizeof(double), GMT_program);
 	*v = (double *) GMT_memory (VNULL, (size_t)(np*np), sizeof(double), GMT_program);
@@ -545,9 +550,10 @@ void allocate_array_space (int np, double **gtg, double **v, double **gtd, doubl
 	*w_model = (double *) GMT_memory (VNULL, (size_t)np, sizeof(double), GMT_program);
 }
 
-void write_output (struct TREND1D_DATA *data, int n_data, char *output_choice, int n_outputs)
+void write_output (struct TREND1D_DATA *data, GMT_LONG n_data, char *output_choice, int n_outputs)
 {
-	int	i, j;
+	GMT_LONG	i;
+	int j;
 	double out[5];
 
 	for (i = 0; i < n_data; i++) {
@@ -589,9 +595,9 @@ void free_the_memory (double *gtg, double *v, double *gtd, double *lambda, doubl
 	GMT_free ((void *)gtg);
 }
 
-void transform_x (struct TREND1D_DATA *data, int n_data, int model_type, double xmin, double xmax)
+void transform_x (struct TREND1D_DATA *data, GMT_LONG n_data, int model_type, double xmin, double xmax)
 {
-	int	i;
+	GMT_LONG	i;
 	double	offset, scale;
 
 	offset = 0.5 * (xmin + xmax);	/* Mid Range  */
@@ -606,9 +612,9 @@ void transform_x (struct TREND1D_DATA *data, int n_data, int model_type, double
 	}
 }
 
-void untransform_x (struct TREND1D_DATA *data, int n_data, int model_type, double xmin, double xmax)
+void untransform_x (struct TREND1D_DATA *data, GMT_LONG n_data, int model_type, double xmin, double xmax)
 {
-	int	i;
+	GMT_LONG	i;
 	double	offset, scale;
 
 	offset = 0.5 * (xmin + xmax);	/* Mid Range  */
@@ -623,9 +629,9 @@ void untransform_x (struct TREND1D_DATA *data, int n_data, int model_type, doubl
 	}
 }
 
-double get_chisq (struct TREND1D_DATA *data, int n_data, int n_model)
+double get_chisq (struct TREND1D_DATA *data, GMT_LONG n_data, int n_model)
 {
-	int	i, nu;
+	GMT_LONG	i, nu;
 	double	chi = 0.0;
 
 
@@ -642,9 +648,9 @@ double get_chisq (struct TREND1D_DATA *data, int n_data, int n_model)
 	return(chi);
 }
 
-void recompute_weights (struct TREND1D_DATA *data, int n_data, double *work, double *scale)
+void recompute_weights (struct TREND1D_DATA *data, GMT_LONG n_data, double *work, double *scale)
 {
-	int	i;
+	GMT_LONG	i;
 	double	k, ksq, rr;
 
 	/* First find median { fabs(data[].r) },
@@ -728,12 +734,13 @@ void load_g_row (double x, int n, double *gr, int m)
 	}
 }
 
-void calc_m_and_r (struct TREND1D_DATA *data, int n_data, double *model, int n_model, int m_type, double *grow)
+void calc_m_and_r (struct TREND1D_DATA *data, GMT_LONG n_data, double *model, int n_model, int m_type, double *grow)
 {
 	/*	model[n_model] holds solved coefficients of m_type model.
 		grow[n_model] is a vector for a row of G matrix.  */
 
-	int	i, j;
+	GMT_LONG	i;
+	int j;
 	for (i = 0; i < n_data; i++) {
 		load_g_row(data[i].x, n_model, grow, m_type);
 		data[i].m = 0.0;
@@ -753,11 +760,12 @@ void move_model_a_to_b (double *model_a, double *model_b, int n_model, double *c
 	*chisq_b = *chisq_a;
 }
 
-void load_gtg_and_gtd (struct TREND1D_DATA *data, int n_data, double *gtg, double *gtd, double *grow, int n_model, int mp, int m_type)
+void load_gtg_and_gtd (struct TREND1D_DATA *data, GMT_LONG n_data, double *gtg, double *gtd, double *grow, int n_model, GMT_LONG mp, int m_type)
 {
    	/* mp is row dimension of gtg  */
 
-	int	i, j, k;
+	GMT_LONG	i;
+	int j, k;
 	double	wy;
 
 	/* First zero the contents for summing:  */
@@ -792,10 +800,11 @@ void load_gtg_and_gtd (struct TREND1D_DATA *data, int n_data, double *gtg, doubl
 	}
 }
 
-void solve_system (double *gtg, double *gtd, double *model, int n_model, int mp, double *lambda, double *v, double *b, double *z, double c_no, int *ir)
+void solve_system (double *gtg, double *gtd, double *model, int n_model, GMT_LONG mp, double *lambda, double *v, double *b, double *z, double c_no, int *ir)
 {
 
-	int	i, j, k, rank = 0, n, m, nrots;
+	int	i, j, k, rank = 0, nrots;
+	GMT_LONG n, m;
 	double	c_test, temp_inverse_ij;
 
 	if (n_model == 1) {
@@ -803,8 +812,8 @@ void solve_system (double *gtg, double *gtd, double *model, int n_model, int mp,
 		*ir = 1;
 	}
 	else {
-		n = n_model;
-		m = mp;
+		n = (GMT_LONG)n_model;
+		m = (GMT_LONG)mp;
 		if(GMT_jacobi(gtg, &n, &m, lambda, v, b, z, &nrots)) {
 			fprintf(stderr,"%s:  Warning:  Matrix Solver Convergence Failure.\n", GMT_program);
 		}
@@ -875,7 +884,7 @@ void GMT_cheb_to_pol (double c[], int n, double a, double b)
 void *New_Trend1d_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct TREND1D_CTRL *C;
 	
-	C = (struct TREND1D_CTRL *) GMT_memory (VNULL, 1, sizeof (struct TREND1D_CTRL), "New_Trend1d_Ctrl");
+	C = (struct TREND1D_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct TREND1D_CTRL), "New_Trend1d_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->C.value = 1.0e06;		/* Condition number for matrix solution  */	
